{-|
Module      : OpenAlex Utils
Description : Textmining Collaborative Platform
Copyright   : (c) CNRS, 2023-present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX
-}
module OpenAlex.Utils where

import qualified Data.Map.Strict as Map
import qualified Data.Text as T
import Protolude


-- | https://docs.openalex.org/api-entities/works/work-object#abstract_inverted_index
-- https://en.wikipedia.org/wiki/Inverted_index
-- For legal reasons, OpenAlex doesn't include abstract full text. But
-- we can reconstruct most of it from the index.
-- The index is of form: { word : [ positions-in-text ] }
reconstructAbstract :: Maybe (Map Text [Int]) -> Text
reconstructAbstract Nothing = ""
reconstructAbstract (Just m) = T.intercalate " " $ snd <$> sort wordPositions
  where
    wordPositions = concatMap (\(word, positions) -> (\pos -> (pos, word)) <$> positions) $ Map.toList m
